@@ -71,12 +71,6 @@ pub struct Screen<'screen> {
     pub context_manager: context::ContextManager<EventProxy>,
 }
 
-// impl<'a> Drop for Screen<'a> {
-//     fn drop(&mut self) {
-//         println!("dropped screen");
-//     }
-// }
-
 pub struct ScreenWindowProperties {
     pub size: winit::dpi::PhysicalSize<u32>,
     pub scale: f64,
@@ -101,8 +95,10 @@ impl Screen<'_> {
         let window_id = window_properties.window_id;
         let theme = window_properties.theme;
 
-        let padding_y_bottom = padding_bottom_from_config(config);
-        let padding_y_top = padding_top_from_config(config);
+        let padding_y_top =
+            padding_top_from_config(&config.navigation, config.padding_y[0], 1);
+        let padding_y_bottom =
+            padding_bottom_from_config(&config.navigation, config.padding_y[1], 1);
 
         let sugarloaf_layout = SugarloafLayout::new(
             size.width as f32,
@@ -206,7 +202,7 @@ impl Screen<'_> {
             window_id,
             0,
             context_manager_config,
-            sugarloaf.layout_next(),
+            sugarloaf.layout(),
             sugarloaf_errors,
         )?;
 
@@ -295,11 +291,14 @@ impl Screen<'_> {
         current_theme: Option<winit::window::Theme>,
         font_library: &rio_backend::sugarloaf::font::FontLibrary,
     ) {
-        let padding_y_bottom = padding_bottom_from_config(config);
-        let padding_y_top = padding_top_from_config(config);
+        let num_tabs = self.ctx().len();
+        let padding_y_top =
+            padding_top_from_config(&config.navigation, config.padding_y[0], num_tabs);
+        let padding_y_bottom =
+            padding_bottom_from_config(&config.navigation, config.padding_y[1], num_tabs);
 
         self.sugarloaf.update_font(font_library);
-        self.sugarloaf.layout_next_mut().recalculate(
+        self.sugarloaf.layout_mut().recalculate(
             config.fonts.size,
             config.line_height,
             config.padding_x,
@@ -307,7 +306,7 @@ impl Screen<'_> {
             padding_y_bottom,
         );
 
-        self.sugarloaf.layout_next_mut().update();
+        self.sugarloaf.layout_mut().update();
         self.state = State::new(config, current_theme);
 
         for context in self.ctx().contexts() {
@@ -327,6 +326,7 @@ impl Screen<'_> {
             self.sugarloaf.set_background_image(image);
         }
 
+        self.update_content();
         self.render();
         self.resize_all_contexts();
     }
@@ -341,6 +341,7 @@ impl Screen<'_> {
 
         self.sugarloaf.update_font_size(action);
 
+        self.update_content();
         self.render();
         self.resize_all_contexts();
     }
@@ -360,6 +361,7 @@ impl Screen<'_> {
     ) -> &mut Self {
         self.sugarloaf.rescale(new_scale);
         self.sugarloaf.resize(new_size.width, new_size.height);
+        self.update_content();
         self.render();
         self.resize_all_contexts();
 
@@ -372,7 +374,7 @@ impl Screen<'_> {
         // the next layout, so once the messenger.send_resize triggers
         // the wakeup from pty it will also trigger a sugarloaf.render()
         // and then eventually a render with the new layout computation.
-        let layout = self.sugarloaf.layout_next();
+        let layout = self.sugarloaf.layout();
         for context in self.ctx().contexts() {
             let mut terminal = context.terminal.lock();
             terminal.resize::<SugarloafLayout>(layout);
@@ -413,7 +415,7 @@ impl Screen<'_> {
 
     #[inline]
     pub fn display_offset(&self) -> usize {
-        let mut terminal = self.ctx().current().terminal.lock();
+        let terminal = self.ctx().current().terminal.lock();
         let display_offset = terminal.display_offset();
         drop(terminal);
         display_offset
@@ -560,6 +562,7 @@ impl Screen<'_> {
                             self.state.set_selection(selection.to_range(&terminal));
                         };
                         drop(terminal);
+                        self.update_content();
                         self.render();
                     }
                     Act::Vi(ViAction::CenterAroundViCursor) => {
@@ -601,10 +604,39 @@ impl Screen<'_> {
                             layout,
                             (
                                 &self.state.get_cursor_state_from_ref(),
-                                self.state.has_blinking_enabled,
+                                self.state.config_has_blinking_enabled,
                             ),
                         );
 
+                        let previous_margin = layout.margin;
+                        let num_tabs = self.ctx().len();
+                        let padding_y_top = padding_top_from_config(
+                            &self.state.navigation.navigation,
+                            self.state.navigation.padding_y[0],
+                            num_tabs,
+                        );
+                        let padding_y_bottom = padding_bottom_from_config(
+                            &self.state.navigation.navigation,
+                            self.state.navigation.padding_y[1],
+                            num_tabs,
+                        );
+
+                        if previous_margin.top_y != padding_y_top
+                            || previous_margin.bottom_y != padding_y_bottom
+                        {
+                            let layout = self.sugarloaf.layout();
+                            self.sugarloaf.layout_mut().recalculate(
+                                layout.font_size,
+                                layout.line_height,
+                                layout.margin.x,
+                                padding_y_top,
+                                padding_y_bottom,
+                            );
+                            self.sugarloaf.layout_mut().update();
+                            self.resize_all_contexts();
+                        }
+
+                        self.update_content();
                         self.render();
                     }
                     Act::TabCloseCurrent => {
@@ -612,11 +644,48 @@ impl Screen<'_> {
 
                         if self.context_manager.config.is_native {
                             self.context_manager.close_current_window(false);
+                            return;
                         } else {
                             // Kill current context will trigger terminal.exit
                             // then RioEvent::Exit and eventually try_close_existent_tab
                             self.context_manager.kill_current_context();
                         }
+
+                        if self.ctx().len() <= 1 {
+                            return;
+                        }
+
+                        let layout = self.sugarloaf.layout();
+                        let previous_margin = layout.margin;
+                        let num_tabs = self.ctx().len().wrapping_sub(1);
+                        let padding_y_top = padding_top_from_config(
+                            &self.state.navigation.navigation,
+                            self.state.navigation.padding_y[0],
+                            num_tabs,
+                        );
+                        let padding_y_bottom = padding_bottom_from_config(
+                            &self.state.navigation.navigation,
+                            self.state.navigation.padding_y[1],
+                            num_tabs,
+                        );
+
+                        if previous_margin.top_y != padding_y_top
+                            || previous_margin.bottom_y != padding_y_bottom
+                        {
+                            let layout = self.sugarloaf.layout();
+                            self.sugarloaf.layout_mut().recalculate(
+                                layout.font_size,
+                                layout.line_height,
+                                layout.margin.x,
+                                padding_y_top,
+                                padding_y_bottom,
+                            );
+                            self.sugarloaf.layout_mut().update();
+                            self.resize_all_contexts();
+                        }
+
+                        self.update_content();
+                        self.render();
                     }
                     Act::Quit => {
                         self.context_manager.quit();
@@ -710,6 +779,7 @@ impl Screen<'_> {
                             self.context_manager.current_mut().terminal.lock();
                         terminal.clear_saved_history();
                         drop(terminal);
+                        self.update_content();
                         self.render();
                     }
                     Act::ToggleFullscreen => self.context_manager.toggle_full_screen(),
@@ -725,20 +795,24 @@ impl Screen<'_> {
                     }
                     Act::SelectTab(tab_index) => {
                         self.context_manager.select_tab(*tab_index);
+                        self.update_content();
                         self.render();
                     }
                     Act::SelectLastTab => {
                         self.context_manager.select_last_tab();
+                        self.update_content();
                         self.render();
                     }
                     Act::SelectNextTab => {
                         self.clear_selection();
                         self.context_manager.switch_to_next();
+                        self.update_content();
                         self.render();
                     }
                     Act::SelectPrevTab => {
                         self.clear_selection();
                         self.context_manager.switch_to_prev();
+                        self.update_content();
                         self.render();
                     }
                     Act::ReceiveChar | Act::None => (),
@@ -992,7 +1066,7 @@ impl Screen<'_> {
             return false;
         }
 
-        let mut terminal = self.context_manager.current().terminal.lock();
+        let terminal = self.context_manager.current().terminal.lock();
         let display_offset = terminal.display_offset();
         let pos = self.mouse_position(display_offset);
         let pos_hyperlink = terminal.grid[pos].hyperlink();
@@ -1193,6 +1267,30 @@ impl Screen<'_> {
         self.sugarloaf.render();
     }
 
+    pub fn update_content(&mut self) {
+        let (rows, cursor, display_offset, has_blinking_enabled) = {
+            let terminal = self.ctx().current().terminal.lock();
+            (
+                terminal.visible_rows(),
+                terminal.cursor(),
+                terminal.display_offset(),
+                terminal.blinking_cursor,
+            )
+        };
+
+        self.context_manager.update_titles();
+        self.state.set_ime(self.ime.preedit());
+
+        self.state.prepare_term(
+            &rows,
+            cursor,
+            &mut self.sugarloaf,
+            &self.context_manager,
+            display_offset as i32,
+            has_blinking_enabled,
+        );
+    }
+
     #[inline]
     pub fn render(&mut self) {
         // If sugarloaf does have pending updates to process then
@@ -1201,40 +1299,12 @@ impl Screen<'_> {
         // let start = std::time::Instant::now();
         // println!("Render time elapsed");
 
-        #[cfg(use_wa)]
-        if self.sugarloaf.dimensions_changed() {
-            self.resize_all_contexts();
-            return;
-        };
-
-        let mut terminal = self.ctx().current().terminal.lock();
-        let visible_rows = terminal.visible_rows();
-        let cursor = terminal.cursor();
-        let display_offset = terminal.display_offset();
-        let has_blinking_enabled = terminal.blinking_cursor;
-        drop(terminal);
-        self.context_manager.update_titles();
-
-        self.state.set_ime(self.ime.preedit());
-
-        self.state.prepare_term(
-            visible_rows,
-            cursor,
-            &mut self.sugarloaf,
-            &self.context_manager,
-            display_offset as i32,
-            has_blinking_enabled,
-        );
-
         self.sugarloaf.render();
 
         // In this case the configuration of blinking cursor is enabled
         // and the terminal also have instructions of blinking enabled
         // TODO: enable blinking for selection after adding debounce (https://github.com/raphamorim/rio/issues/437)
-        if self.state.has_blinking_enabled
-            && has_blinking_enabled
-            && self.selection_is_empty()
-        {
+        if self.state.has_blinking_enabled() && self.selection_is_empty() {
             self.context_manager.schedule_render_on_route(800);
         }
 
@@ -1268,7 +1338,7 @@ impl Screen<'_> {
 
     #[inline]
     pub fn mouse_report(&mut self, button: u8, state: ElementState) {
-        let mut terminal = self.ctx().current().terminal.lock();
+        let terminal = self.ctx().current().terminal.lock();
         let display_offset = terminal.display_offset();
         let mode = terminal.mode();
         drop(terminal);
