@@ -73,14 +73,6 @@ impl Sequencer {
                         payload, window_id, ..
                     }) => {
                         match payload {
-                            RioEventType::Rio(RioEvent::Wakeup) => {
-                                // Emitted when the application has been resumed.
-                                if let Some(route) =
-                                    self.router.routes.get_mut(&window_id)
-                                {
-                                    route.window.winit_window.request_redraw();
-                                }
-                            }
                             RioEventType::Rio(RioEvent::Render) => {
                                 if let Some(route) =
                                     self.router.routes.get_mut(&window_id)
@@ -90,6 +82,7 @@ impl Sequencer {
                                     {
                                         return;
                                     }
+                                    route.window.screen.update_content();
                                     route.window.winit_window.request_redraw();
                                 }
                             }
@@ -97,9 +90,16 @@ impl Sequencer {
                                 if let Some(route) =
                                     self.router.routes.get_mut(&window_id)
                                 {
+                                    if self.config.renderer.disable_unfocused_render
+                                        && !route.window.is_focused
+                                    {
+                                        return;
+                                    }
+
                                     if route_id
                                         == route.window.screen.ctx().current_route()
                                     {
+                                        route.window.screen.update_content();
                                         route.window.winit_window.request_redraw();
                                     }
                                 }
@@ -199,6 +199,7 @@ impl Sequencer {
                                 {
                                     if self.config.confirm_before_quit {
                                         route.confirm_quit();
+                                        route.window.screen.update_content();
                                         route.redraw();
                                     } else {
                                         route.quit();
@@ -599,6 +600,7 @@ impl Sequencer {
                                         &self.router.font_library,
                                     );
 
+                                    route.window.screen.update_content();
                                     route.redraw();
                                 }
                             }
@@ -754,7 +756,8 @@ impl Sequencer {
                                             route.window.screen.on_left_click(pos);
                                         }
 
-                                        route.window.winit_window.request_redraw();
+                                        route.window.screen.update_content();
+                                        route.redraw();
                                     }
                                     route.window.screen.process_mouse_bindings(button);
                                 }
@@ -1077,6 +1080,7 @@ impl Sequencer {
                                         != preedit.as_ref()
                                     {
                                         route.window.screen.ime.set_preedit(preedit);
+                                        route.window.screen.update_content();
                                         route.redraw();
                                     }
                                 }
@@ -1113,6 +1117,7 @@ impl Sequencer {
                             route.window.is_focused = focused;
 
                             if has_regained_focus {
+                                route.window.screen.update_content();
                                 route.redraw();
                             }
 
@@ -1189,6 +1194,7 @@ impl Sequencer {
                                 .window
                                 .screen
                                 .set_scale(scale, route.window.winit_window.inner_size());
+                            route.window.screen.update_content();
                             route.redraw();
                         }
                     }
@@ -1233,7 +1239,6 @@ impl Sequencer {
                                     route.window.screen.render_welcome();
                                 }
                                 RoutePath::Terminal => {
-                                    // Notify winit that we're about to present.
                                     route.window.screen.render();
                                 }
                                 RoutePath::ConfirmQuit => {
