@@ -13,13 +13,18 @@ use rio_backend::config::colors::{
     AnsiColor, ColorArray, Colors, NamedColor,
 };
 use rio_backend::config::Config;
-use rio_backend::sugarloaf::{Sugar, SugarCursor, SugarDecoration, SugarStyle};
-use rio_backend::sugarloaf::{SugarGraphic, Sugarloaf};
+use rio_backend::sugarloaf::{
+    Content, ContentBuilder, FragmentStyle, FragmentStyleDecoration, Stretch, Style,
+    SugarCursor, Sugarloaf, UnderlineInfo, UnderlineShape, Weight,
+};
 use std::collections::HashMap;
 use std::time::{Duration, Instant};
 #[cfg(not(use_wa))]
 use winit::window::Theme;
 
+use rustc_hash::FxHashMap;
+use unicode_width::UnicodeWidthChar;
+
 struct Cursor {
     state: CursorState,
     content: char,
@@ -36,16 +41,18 @@ pub struct State {
     pub named_colors: Colors,
     font_size: f32,
     pub colors: List,
-    navigation: ScreenNavigation,
+    pub navigation: ScreenNavigation,
     cursor: Cursor,
     pub selection_range: Option<SelectionRange>,
-    pub has_blinking_enabled: bool,
+    pub config_has_blinking_enabled: bool,
+    term_has_blinking_enabled: bool,
     pub is_blinking: bool,
     ignore_selection_fg_color: bool,
     // Dynamic background keep track of the original bg color and
     // the same r,g,b with the mutated alpha channel.
     pub dynamic_background: ([f32; 4], wgpu::Color, bool),
     hyperlink_range: Option<SelectionRange>,
+    width_cache: FxHashMap<char, f32>,
 }
 
 impl State {
@@ -119,20 +126,21 @@ impl State {
             is_vi_mode_enabled: false,
             is_blinking: false,
             last_typing: None,
-            has_blinking_enabled: config.blinking_cursor,
+            config_has_blinking_enabled: config.blinking_cursor,
+            term_has_blinking_enabled: false,
             ignore_selection_fg_color: config.ignore_selection_fg_color,
             colors,
             navigation: ScreenNavigation::new(
-                config.navigation.mode,
+                config.navigation.clone(),
                 [
+                    named_colors.foreground,
+                    named_colors.bar,
                     named_colors.tabs,
                     named_colors.tabs_active,
-                    named_colors.foreground,
+                    named_colors.tabs_active_highlight,
                 ],
                 color_automation,
-                0.0,
-                0.0,
-                0.0,
+                config.padding_y,
             ),
             font_size: config.fonts.size,
             selection_range: None,
@@ -144,9 +152,15 @@ impl State {
                 content_ref: config.cursor,
                 state: CursorState::new(config.cursor),
             },
+            width_cache: FxHashMap::default(),
         }
     }
 
+    #[inline]
+    pub fn has_blinking_enabled(&self) -> bool {
+        self.config_has_blinking_enabled && self.term_has_blinking_enabled
+    }
+
     #[inline]
     pub fn get_cursor_state_from_ref(&self) -> CursorState {
         CursorState::new(self.cursor.content_ref)
@@ -168,7 +182,7 @@ impl State {
     }
 
     #[inline]
-    fn create_sugar(&self, square: &Square) -> Sugar {
+    fn create_style(&mut self, square: &Square) -> (FragmentStyle, char) {
         let flags = square.flags;
 
         let mut foreground_color = self.compute_color(&square.fg, flags);
@@ -180,43 +194,29 @@ impl State {
             square.c
         };
 
-        let style = match (
+        let width = if let Some(w) = self.width_cache.get(&content) {
+            *w
+        } else {
+            let w = square.c.width().unwrap_or(1) as f32;
+            self.width_cache.insert(square.c, w);
+            w
+        };
+
+        let font_attrs = match (
             flags.contains(Flags::ITALIC),
             flags.contains(Flags::BOLD_ITALIC),
             flags.contains(Flags::BOLD),
         ) {
-            (true, _, _) => SugarStyle::Italic,
-            (_, true, _) => SugarStyle::BoldItalic,
-            (_, _, true) => SugarStyle::Bold,
-            _ => SugarStyle::Disabled,
+            (true, _, _) => (Stretch::NORMAL, Weight::NORMAL, Style::Italic),
+            (_, true, _) => (Stretch::NORMAL, Weight::BOLD, Style::Italic),
+            (_, _, true) => (Stretch::NORMAL, Weight::BOLD, Style::Normal),
+            _ => (Stretch::NORMAL, Weight::NORMAL, Style::Normal),
         };
 
         if flags.contains(Flags::INVERSE) {
             std::mem::swap(&mut background_color, &mut foreground_color);
         }
 
-        let mut decoration = SugarDecoration::Disabled;
-        if flags.contains(Flags::UNDERLINE) {
-            decoration = SugarDecoration::Underline;
-        } else if flags.contains(Flags::STRIKEOUT) {
-            decoration = SugarDecoration::Strikethrough;
-        } else if flags.contains(Flags::DOUBLE_UNDERLINE) {
-            decoration = SugarDecoration::DoubleUnderline;
-        } else if flags.contains(Flags::DOTTED_UNDERLINE) {
-            decoration = SugarDecoration::DottedUnderline;
-        } else if flags.contains(Flags::DASHED_UNDERLINE) {
-            decoration = SugarDecoration::DashedUnderline;
-        } else if flags.contains(Flags::UNDERCURL) {
-            decoration = SugarDecoration::CurlyUnderline;
-        }
-
-        let mut decoration_color = None;
-        if decoration != SugarDecoration::Disabled {
-            if let Some(color) = square.underline_color() {
-                decoration_color = Some(self.compute_color(&color, square.flags));
-            }
-        };
-
         let background_color = if self.dynamic_background.2
             && background_color[0] == self.dynamic_background.0[0]
             && background_color[1] == self.dynamic_background.0[1]
@@ -227,36 +227,92 @@ impl State {
             Some(background_color)
         };
 
-        Sugar {
+        let (decoration, decoration_color) = self.compute_decoration(square, false);
+
+        (
+            FragmentStyle {
+                width,
+                color: foreground_color,
+                background_color,
+                font_attrs,
+                decoration,
+                decoration_color,
+                ..FragmentStyle::default()
+            },
             content,
-            repeated: 0,
-            foreground_color,
-            background_color,
-            decoration_color,
-            style,
-            decoration,
-            media: None,
-            cursor: SugarCursor::Disabled,
+        )
+    }
+
+    fn compute_decoration(
+        &self,
+        square: &Square,
+        skip_underline: bool,
+    ) -> (Option<FragmentStyleDecoration>, Option<[f32; 4]>) {
+        let mut decoration = None;
+        let mut decoration_color = None;
+
+        if square.flags.contains(Flags::UNDERLINE) {
+            if !skip_underline {
+                decoration = Some(FragmentStyleDecoration::Underline(UnderlineInfo {
+                    offset: -2.0,
+                    size: 2.0,
+                    is_doubled: false,
+                    shape: UnderlineShape::Regular,
+                }));
+            }
+        } else if square.flags.contains(Flags::STRIKEOUT) {
+            decoration = Some(FragmentStyleDecoration::Strikethrough);
+        } else if square.flags.contains(Flags::DOUBLE_UNDERLINE) {
+            decoration = Some(FragmentStyleDecoration::Underline(UnderlineInfo {
+                offset: -4.0,
+                size: 1.0,
+                is_doubled: true,
+                shape: UnderlineShape::Regular,
+            }));
+        } else if square.flags.contains(Flags::DOTTED_UNDERLINE) {
+            decoration = Some(FragmentStyleDecoration::Underline(UnderlineInfo {
+                offset: -2.0,
+                size: 2.0,
+                is_doubled: false,
+                shape: UnderlineShape::Dotted,
+            }));
+        } else if square.flags.contains(Flags::DASHED_UNDERLINE) {
+            decoration = Some(FragmentStyleDecoration::Underline(UnderlineInfo {
+                offset: -2.0,
+                size: 2.0,
+                is_doubled: false,
+                shape: UnderlineShape::Dashed,
+            }));
+        } else if square.flags.contains(Flags::UNDERCURL) {
+            decoration = Some(FragmentStyleDecoration::Underline(UnderlineInfo {
+                offset: -2.0,
+                size: 1.0,
+                is_doubled: false,
+                shape: UnderlineShape::Curly,
+            }));
         }
+
+        if decoration.is_some() {
+            if let Some(color) = square.underline_color() {
+                decoration_color = Some(self.compute_color(&color, square.flags));
+            }
+        };
+
+        (decoration, decoration_color)
     }
 
     #[inline]
-    fn set_hyperlink_in_sugar(&self, mut sugar: Sugar) -> Sugar {
-        sugar.decoration = SugarDecoration::Underline;
-        sugar
-    }
-
-    #[inline]
-    fn create_sugar_line(
+    fn create_line(
         &mut self,
-        sugarloaf: &mut Sugarloaf,
+        content_builder: &mut ContentBuilder,
         row: &Row<Square>,
         has_cursor: bool,
         current_line: pos::Line,
     ) {
-        sugarloaf.start_line();
-
         let columns: usize = row.len();
+        let mut content = String::default();
+        let mut last_style = FragmentStyle::default();
+
         for column in 0..columns {
             let square = &row.inner[column];
 
@@ -265,56 +321,67 @@ impl State {
             }
 
             if square.flags.contains(Flags::GRAPHICS) {
-                sugarloaf.insert_on_current_line(&self.create_graphic_sugar(square));
+                // &self.create_graphic_sugar(square);
                 continue;
             }
 
-            if has_cursor && column == self.cursor.state.pos.col {
-                sugarloaf.insert_on_current_line(&self.create_cursor(square));
-            } else if self.hyperlink_range.is_some()
+            let (mut style, square_content) =
+                if has_cursor && column == self.cursor.state.pos.col {
+                    self.create_cursor_style(square)
+                } else {
+                    self.create_style(square)
+                };
+
+            if self.hyperlink_range.is_some()
                 && square.hyperlink().is_some()
                 && self
                     .hyperlink_range
                     .unwrap()
                     .contains(pos::Pos::new(current_line, pos::Column(column)))
             {
-                let sugar = self.create_sugar(square);
-                sugarloaf.insert_on_current_line(&self.set_hyperlink_in_sugar(sugar));
+                style.decoration =
+                    Some(FragmentStyleDecoration::Underline(UnderlineInfo {
+                        offset: -1.0,
+                        size: -1.0,
+                        is_doubled: false,
+                        shape: UnderlineShape::Regular,
+                    }));
             } else if self.selection_range.is_some()
                 && self
                     .selection_range
                     .unwrap()
                     .contains(pos::Pos::new(current_line, pos::Column(column)))
             {
-                let content = if square.c == '\t' || square.flags.contains(Flags::HIDDEN)
-                {
-                    ' '
+                style.color = if self.ignore_selection_fg_color {
+                    self.compute_color(&square.fg, square.flags)
                 } else {
-                    square.c
+                    self.named_colors.selection_foreground
                 };
-
-                let selected_sugar = Sugar {
-                    content,
-                    foreground_color: if self.ignore_selection_fg_color {
-                        self.compute_color(&square.fg, square.flags)
-                    } else {
-                        self.named_colors.selection_foreground
-                    },
-                    background_color: Some(self.named_colors.selection_background),
-                    ..Sugar::default()
-                };
-                sugarloaf.insert_on_current_line(&selected_sugar);
-            } else {
-                sugarloaf.insert_on_current_line(&self.create_sugar(square));
+                style.background_color = Some(self.named_colors.selection_background);
             }
 
+            if last_style != style {
+                if !content.is_empty() {
+                    content_builder.add_text(&content, last_style);
+                }
+
+                content.clear();
+                last_style = style;
+            }
+
+            content.push(square_content);
+
             // Render last column and break row
             if column == (columns - 1) {
+                if !content.is_empty() {
+                    content_builder.add_text(&content, last_style);
+                }
+
                 break;
             }
         }
 
-        sugarloaf.finish_line();
+        content_builder.finish_line();
     }
 
     #[inline]
@@ -457,76 +524,95 @@ impl State {
         }
     }
 
-    #[inline]
-    fn create_graphic_sugar(&self, square: &Square) -> Sugar {
-        let media = &square.graphics().unwrap()[0].texture;
-        Sugar {
-            media: Some(SugarGraphic {
-                id: media.id,
-                width: media.width,
-                height: media.height,
-            }),
-            ..Sugar::default()
-        }
-    }
+    // #[inline]
+    // #[allow(dead_code)]
+    // fn create_graphic_sugar(&self, square: &Square) -> Sugar {
+    //     let media = &square.graphics().unwrap()[0].texture;
+    //     Sugar {
+    //         media: Some(SugarGraphic {
+    //             id: media.id,
+    //             width: media.width,
+    //             height: media.height,
+    //         }),
+    //         ..Sugar::default()
+    //     }
+    // }
 
     #[inline]
-    fn create_sugar_cursor(&self) -> SugarCursor {
-        let color = if !self.is_vi_mode_enabled {
-            self.named_colors.cursor
-        } else {
-            self.named_colors.vi_cursor
-        };
-
-        match self.cursor.state.content {
-            CursorShape::Block => SugarCursor::Block(color),
-            CursorShape::Underline => SugarCursor::Underline(color),
-            CursorShape::Beam => SugarCursor::Caret(color),
-            CursorShape::Hidden => SugarCursor::Disabled,
-        }
-    }
-
-    #[inline]
-    fn create_cursor(&self, square: &Square) -> Sugar {
-        let style = match (
+    fn create_cursor_style(&self, square: &Square) -> (FragmentStyle, char) {
+        let font_attrs = match (
             square.flags.contains(Flags::ITALIC),
             square.flags.contains(Flags::BOLD_ITALIC),
             square.flags.contains(Flags::BOLD),
         ) {
-            (true, _, _) => SugarStyle::Italic,
-            (_, true, _) => SugarStyle::BoldItalic,
-            (_, _, true) => SugarStyle::Bold,
-            _ => SugarStyle::Disabled,
+            (true, _, _) => (Stretch::NORMAL, Weight::NORMAL, Style::Italic),
+            (_, true, _) => (Stretch::NORMAL, Weight::BOLD, Style::Italic),
+            (_, _, true) => (Stretch::NORMAL, Weight::BOLD, Style::Normal),
+            _ => (Stretch::NORMAL, Weight::NORMAL, Style::Normal),
         };
 
-        let mut foreground_color = self.compute_color(&square.fg, square.flags);
+        let mut color = self.compute_color(&square.fg, square.flags);
         let mut background_color = self.compute_bg_color(square);
+        // If IME is enabled we get the current content to cursor
+        let content = if self.is_ime_enabled {
+            self.cursor.content
+        } else {
+            square.c
+        };
 
         if square.flags.contains(Flags::INVERSE) {
-            std::mem::swap(&mut background_color, &mut foreground_color);
-        }
-
-        let mut sugar = Sugar {
-            content: square.c,
-            foreground_color,
-            background_color: Some(background_color),
-            cursor: self.create_sugar_cursor(),
-            style,
-            ..Sugar::default()
-        };
-
-        // If IME is enabled we get the current content to cursor
-        if self.is_ime_enabled {
-            sugar.content = self.cursor.content;
+            std::mem::swap(&mut background_color, &mut color);
         }
 
         // If IME is enabled or is a block cursor, put background color
         // when cursor is over the character
         if self.is_ime_enabled || self.cursor.state.content == CursorShape::Block {
-            sugar.foreground_color = self.named_colors.background.0;
+            color = self.named_colors.background.0;
         }
 
-        sugar
+        let mut style = FragmentStyle {
+            color,
+            background_color: Some(background_color),
+            font_attrs,
+            ..FragmentStyle::default()
+        };
+
+        let cursor_color = if !self.is_vi_mode_enabled {
+            self.named_colors.cursor
+        } else {
+            self.named_colors.vi_cursor
+        };
+
+        let mut has_underline_cursor = false;
+
+        match self.cursor.state.content {
+            CursorShape::Underline => {
+                style.decoration =
+                    Some(FragmentStyleDecoration::Underline(UnderlineInfo {
+                        offset: -1.0,
+                        size: -1.0,
+                        is_doubled: false,
+                        shape: UnderlineShape::Regular,
+                    }));
+                style.decoration_color = Some(cursor_color);
+
+                has_underline_cursor = true;
+            }
+            CursorShape::Block => {
+                style.cursor = SugarCursor::Block(cursor_color);
+            }
+            CursorShape::Beam => {
+                style.cursor = SugarCursor::Caret(cursor_color);
+            }
+            CursorShape::Hidden => {}
+        }
+
+        let (decoration, decoration_color) =
+            self.compute_decoration(square, has_underline_cursor);
+        style.decoration = decoration;
+        style.decoration_color = decoration_color;
+
+        (style, content)
     }
 
     #[inline]
@@ -556,22 +642,23 @@ impl State {
     #[inline]
     pub fn prepare_term(
         &mut self,
-        rows: Vec<Row<Square>>,
+        rows: &[Row<Square>],
         cursor: CursorState,
         sugarloaf: &mut Sugarloaf,
         context_manager: &crate::context::ContextManager<rio_backend::event::EventProxy>,
         display_offset: i32,
         has_blinking_enabled: bool,
     ) {
-        let layout = sugarloaf.layout_next();
+        let layout = sugarloaf.layout();
         self.cursor.state = cursor;
         let mut is_cursor_visible = self.cursor.state.is_visible();
 
         self.font_size = layout.font_size;
+        self.term_has_blinking_enabled = has_blinking_enabled;
 
         // Only blink cursor if does not contain selection
         let has_selection = self.selection_range.is_some();
-        if !has_selection && self.has_blinking_enabled && has_blinking_enabled {
+        if !has_selection && self.has_blinking_enabled() {
             let mut should_blink = true;
             if let Some(last_typing_time) = self.last_typing {
                 if last_typing_time.elapsed() < Duration::from_secs(1) {
@@ -585,16 +672,20 @@ impl State {
             }
         }
 
+        let mut content_builder = Content::builder();
+
         for (i, row) in rows.iter().enumerate() {
             let has_cursor = is_cursor_visible && self.cursor.state.pos.row == i;
-            self.create_sugar_line(
-                sugarloaf,
+            self.create_line(
+                &mut content_builder,
                 row,
                 has_cursor,
                 pos::Line((i as i32) - display_offset),
             );
         }
 
+        sugarloaf.set_content(content_builder.build());
+
         self.navigation.content(
             (layout.width, layout.height),
             layout.dimensions.scale,
@@ -604,16 +695,6 @@ impl State {
             context_manager.len(),
         );
 
-        sugarloaf.append_rects(self.navigation.rects.to_owned());
-
-        for text in self.navigation.texts.iter() {
-            sugarloaf.text(
-                text.position,
-                text.content.to_owned(),
-                text.font_size,
-                text.color,
-                true,
-            );
-        }
+        sugarloaf.set_objects(self.navigation.objects.clone());
     }
 }
