@@ -110,16 +110,6 @@ pub fn default_disable_ctlseqs_alt() -> bool {
 
 pub fn default_config_file_content() -> String {
     r#"
-# Editor
-#
-# Default editor is "vi".
-#
-# Whenever the key binding `OpenConfigEditor` is triggered it will
-# use the value of the editor along with the rio configuration path.
-# [editor]
-# program = "vi"
-# args = []
-
 # Cursor
 #
 # Default cursor is Block
@@ -215,6 +205,16 @@ blinking-cursor = false
 #
 # confirm-before-quit = false
 
+# Editor
+#
+# Default editor is "vi".
+#
+# Whenever the key binding `OpenConfigEditor` is triggered it will
+# use the value of the editor along with the rio configuration path.
+# [editor]
+# program = "vi"
+# args = []
+
 # Window configuration
 #
 # • width - define the initial window width.
@@ -352,9 +352,9 @@ blinking-cursor = false
 #   • CollapsedTab
 #   • BottomTab
 #   • TopTab
-#   • Breadcrumb
 #   • Plain
 #
+# "hide-if-single" - Hide navigation UI if is single.
 # "clickable" - Enable click on tabs to switch.
 # "use-current-path" - Use same path whenever a new tab is created (Note: requires `use-fork` to be set to false).
 # "color-automation" - Set a specific color for the tab whenever a specific program is running, or in a specific directory.
@@ -363,6 +363,7 @@ blinking-cursor = false
 # [navigation]
 # mode = "CollapsedTab"
 # clickable = false
+# hide-if-single = true
 # use-current-path = false
 # color-automation = []
 
