@@ -4,6 +4,8 @@ pub mod font;
 pub mod layout;
 mod sugarloaf;
 
+pub use swash::{Stretch, Style, Weight};
+
 pub use crate::sugarloaf::{
     compositors::SugarCompositors,
     graphics::{
@@ -13,3 +15,8 @@ pub use crate::sugarloaf::{
     Sugarloaf, SugarloafErrors, SugarloafRenderer, SugarloafWindow, SugarloafWindowSize,
     SugarloafWithErrors,
 };
+pub use components::rect::Rect;
+pub use layout::{
+    Content, ContentBuilder, FragmentStyle, FragmentStyleDecoration, UnderlineInfo,
+    UnderlineShape,
+};
